from numeric import *
from time import time
from kd_tree import kd_search


class Curve(object):
    def __init__(self):
        self.points = []
        self.segments = []
        self.closed = False

    def add(self, segment):
        segment.curve = self
        self.segments.append(segment)


class Segment(object):
    def __init__(self, p0, p1, face=None):
        self.p0 = p0
        self.p1 = p1
        p0.segments.append(self)
        p1.segments.append(self)
        if face is None:
            self.face = common_face(p0.edge, p1.edge)
        else:
            self.face = face
        self.intersections = []


class Point(object):
    def __init__(self, v, edge=None):
        self.v = v
        self.edge = edge
        self.segments = []


def common_face(e1, e2):
    return (set(e1.faces) & set(e2.faces)).pop()


# traverse edges by adjacent faces
def get_curve_piece(face, edge, stamp):
    piece = []
    while True:
        next_edge = get_next_edge(face, edge, stamp)
        if next_edge.visit == stamp:
            return piece + [Point(next_edge.p, next_edge)]
        else:
            next_edge.visit = stamp
        if len(next_edge.faces) == 1:
            return piece + [Point(next_edge.p, next_edge)]
        else:
            next_face = get_next_face(next_edge, face)
            piece += [Point(next_edge.p, next_edge)]
            face, edge = next_face, next_edge


def get_next_edge(f, e, stamp):
    return [ne for ne in f.edges if ne.mark == stamp and ne.id != e.id][0]


def get_next_face(edge, face):
    return [nf for nf in edge.faces if nf.id != face.id][0]


def visible(ray, mesh, face):
    first_face = kd_search(ray, mesh)

    # TODO: this may be wrong
    if first_face is None:
        return True

    adjacency = set()
    for v in first_face.vertices:
        adjacency |= set(v.faces)

    if face in adjacency:
        return True
    else:
        return False


def find_intersections(curves):
    t0 = time()
    segments = []
    for curve in curves:
        for i in range(len(curve.points)-1):
            segment = Segment(curve.points[i], curve.points[i+1])
            segment.curve = curve
            curve.segments.append(segment)
            segments.append(segment)
    print 'segments:', len(segments), time()-t0

    t0 = time()
    endpoints = []
    for curve in curves:
        for point in curve.points:
            endpoints.append( (point.w[0], point) )
    endpoints.sort()
    print 'endpoints:', time()-t0

    t0 = time()
    sweep = set()
    while len(endpoints) > 0:
        point = endpoints.pop()[1]
        for segment in point.segments:
            if segment in sweep:
                sweep.remove(segment)
            else:
                test_segment(segment, sweep)
                sweep.add(segment)
    print 'intersections:', time()-t0


def test_segment(segment, sweep):
    for sweep_segment in sweep:
        t = intersect(segment, sweep_segment)
        if t is not None:
            segment.intersections.append(t[0])
            sweep_segment.intersections.append(t[1])


def intersect(segA, segB):
    common = set([segA.p0, segA.p1]) & set([segB.p0, segB.p1])
    if len(common) > 0: return None

    conv = lambda p: array(p.w[0:2])
    Ap0, Ap1 = conv(segA.p0), conv(segA.p1)
    Bp0, Bp1 = conv(segB.p0), conv(segB.p1)
    p, v = Ap0, Ap1-Ap0
    q, w = Bp0, Bp1-Bp0
    # linear system
    A = array([v, -w]).T
    b = q-p
    try:
        t, s = solve(A, b)
        if t>0 and s>0 and t<1 and s<1:
            return t, s
        else:
            return None
    except:
        return None


def split_curves(curves):
    newcurves = []

    for curve in curves:
        while curve is not None:
            piece, curve = split_curve(curve)
            newcurves.append(piece)

    return newcurves


def split_curve(curve):
    newcurve = Curve()
    i = 0
    for segment in curve.segments:
        if len(segment.intersections) > 0:
            break
        else:
            newcurve.segments.append(segment)
            i += 1

    if i >= len(curve.segments):
        return newcurve, None
    else:
        seg1, seg2 = split_segment(segment)
        seg1.curve = newcurve
        seg2.curve = curve
        newcurve.segments.append(seg1)
        curve.segments = [seg2] + curve.segments[i+1:]
        return newcurve, curve


def split_segment(segment):
    segment.intersections.sort()
    t = segment.intersections.pop(0)
    p = segment.p0.v
    v = segment.p1.v - segment.p0.v
    r = p + t*v
    point = Point(r)
    q = segment.p0.w
    w = segment.p1.w - segment.p1.w
    point.w = q + t*w
    seg1 = Segment(segment.p0, point, segment.face)
    seg2 = Segment(point, segment.p1, segment.face)
    return seg1, seg2

