from render import *
from draw import *
from numeric import *


def compute_depth_map(obj, cam, wait=None):
    painter = Painter(cam.size)
    render_depth_map(obj, cam, painter, wait)
    dmap = normalize_depth_map(painter.zbuffer, painter.zmax)
    return dmap

def compute_normal_map(obj, cam, wait=None):
    painter = Painter(cam.size)
    render_normal_map(obj, cam, painter, wait)
    nmap = normalize_normal_map(painter.nbuffer)
    return nmap

def detect_dmap_edges(dmap, hard=True):
    TAU = 0.3
    edges = detect_edges(dmap, TAU, hard)
    return edges

def detect_nmap_edges(nmap, hard=True):
    TAU = 0.3
    Is = nmap[:,:,0], nmap[:,:,1], nmap[:,:,2]
    f1, f2, f3 = [detect_edges(I, TAU, hard) for I in Is]
    edges = (f1+f2+f3)/3
    return edges

###

def normalize_depth_map(dmap, zmax):
    m = dmap.min()
    dmap -= m
    T = (zmax-m)*1.2
    dmap /= T
    dmap = clip(dmap, 0.0, 1.0)
    return dmap

def normalize_normal_map(nmap):
    nmap *= 0.5
    nmap += 0.5
    return nmap

def draw_gray_map(painter, dmap):
    for k in range(3):
        painter.buffer[:,:,k] = 255.0*dmap

def draw_color_map(painter, nmap):
    painter.buffer[:,:] = 255.0*nmap

###

p5 = array([0.03647, 0.248968, 0.429123, 0.248968, 0.03647])
d5 = array([-0.108385, -0.280349, 0.0, 0.280349, 0.108385])

def sobel_filter(I):
    Ix = array([convolve(row, p5, 'same') for row in I])
    Ixy = array([convolve(col, d5, 'same') for col in Ix.T]).T
    Iy = array([convolve(row, d5, 'same') for row in I])
    Iyx = array([convolve(col, p5, 'same') for col in Iy.T]).T
    dI = sqrt(abs(Ixy) + abs(Iyx))
    return dI

def detect_edges(I, tau=0.2, hard=True):
    fI = sobel_filter(I)
    if hard:
        rI = where(fI > tau, 0.0, 1.0)
    else:
        rI = where(fI > tau, 1-fI, 1.0)
    return rI

