from render import make_transform, make_inverse_transform, project
from kd_tree import kd_search
from ray import *
from curve import *
import sdl


def find_silhouette_curves(mesh, cam, wait=None):
    T = make_transform(cam)
    Ti = make_inverse_transform(cam)
    eye = cam.center

    # calculate dot produtcs at vertices
    t0 = time()
    for v in mesh.vertices:
        e = v.vector - eye
        e /= norm(e)
        n = v.normal
        v.d = dot(n, e)
    print 'dot product:', time()-t0, 'secs'

    # time stamp
    stamp = int(time()*1000)

    # scan edges for signal transitions
    t0 = time()
    signal = lambda d: d >= 0.0
    marked_edges = {}
    for e in mesh.edges.values():
        vi, vj = e.v0, e.v1
        di, dj = vi.d, vj.d
        if signal(di) == signal(dj): continue
        marked_edges[e.id] = e
        dij = abs(di) + abs(dj)
        p = abs(dj/dij)*vi.vector + abs(di/dij)*vj.vector
        e.p = p
        e.mark = stamp
    print 'marked edges:', len(marked_edges), time()-t0, 'secs'

    # tie edges silhuette points
    t0 = time()
    curves = []
    while len(marked_edges) > 0:
        id, e = marked_edges.popitem()
        if e.visit == stamp: continue
        e.visit = stamp
        curve = Curve()
        curve.points += [Point(e.p, e)]
        for f in e.faces:
            if get_next_edge(f, e, stamp).visit == stamp: continue
            piece = get_curve_piece(f, e, stamp)
            curve.points.reverse()
            curve.points += piece
        curves += [curve]
    print 'silhouette curves:', len(curves), len(marked_edges), time()-t0, 'secs'

    out = lambda w: w[0] < 0 or w[1] < 0 or w[0] > cam.size[0]-1 or w[1] > cam.size[1]-1

    for curve in curves:
        for point in curve.points:
            point.w = array(project(T, point.v)[0:2])

    find_intersections(curves)
    curves = split_curves(curves)
    print 'new curves:', len(curves)

    return curves


def show_curves(mesh, curves, surface, eye):
    t0 = time()

    def is_visible(curve, i):
        seg = curve.segments[i]
        ray = Ray(eye, (seg.p0.v + seg.p1.v)/2 - eye)
        return visible(ray, mesh, seg.face)

    for curve in curves:
        n = len(curve.segments)

        if is_visible(curve, n/2):
            visib = True
        else:
            if n < 5:
                visib = False
            else:
                if is_visible(curve, 1) and is_visible(curve, -2):
                    visib = True
                else:
                    visib = False

        if visib:
            points = [seg.p0.w.tolist() for seg in curve.segments]
            points.append(curve.segments[-1].p1.w)
            sdl.draw.aalines(surface, (0, 0, 0), False, points)

    print 'silhouette drawed:', time()-t0, 'secs'
    sdl.display.flip()

