import scene
from numeric import *


class Vertex(object):
    def __init__(self, vector, id):
        self.id = id
        try:
            self.vector = array(vector, float)
        except:
            print 'bad vertex definition: %s' % (str(vector))
            raise
        self.faces = []
        self.neighbors = []

    def __repr__(self):
        return self.vector.__repr__()

    def add_face(self, face):
        if face not in self.faces:
            self.faces.append(face)

    def add_neighbors(self, neighbors):
        for neighbor in neighbors:
            if neighbor not in self.neighbors:
                self.neighbors.append(neighbor)


class Face(object):
    def __init__(self, vertices, id):
        self.id = id
        self.vertices = vertices
        self.edges = []
        self.visit = 0

    def get_vectors(self):
        return [vertex.vector for vertex in self.vertices]
    vectors = property(get_vectors)

    def __getitem__(self, name):
        return self.vertices.__getitem__(name)

    def __len__(self):
        return self.vertices.__len__()

    def __repr__(self):
        return self.vertices.__repr__()

    def add_edge(self, e):
        if e not in self.edges:
            self.edges += [e]


class Edge(object):
    MAX = 10**8

    def __init__(self, v0, v1):
        self.id = Edge.hash(v0, v1)
        self.visit = 0
        self.mark = 0
        self.faces = []
        if v0.id < v1.id:
            self.v0, self.v1 = v0, v1
        else:
            self.v0, self.v1 = v1, v0

    def add_face(self, f):
        if f not in self.faces:
            self.faces += [f]

    @staticmethod
    def hash_max(max):
        Edge.MAX = max

    @staticmethod
    def hash(v0, v1):
        ids = v0.id, v1.id
        m, M = min(ids), max(ids)
        return m + M*Edge.MAX


class Mesh(scene.Object):
    def __init__(self, vertices=None, faces=None):
        self.vertices = []
        self.faces = []
        self.edges = {}

        if vertices is not None:
            idx = xrange(len(vertices))
            self.vertices = [Vertex(v, i) for v, i in zip(vertices, idx)]

        if faces is not None:
            idx = xrange(len(faces))
            self.faces = [Face([self.vertices[j] for j in face], i) for face, i in zip(faces, idx)]

    def get_vectors(self):
        return [vertex.vector for vertex in self.vertices]
    vectors = property(get_vectors)

    @staticmethod
    def fromBuffer(buf):
        header = buf.readline().strip()
        if header != 'OFF': raise 'error: bad OFF file'
        mesh = Mesh()
        vertices, faces, z = map(int, (buf.readline().split()))
        for id in xrange(vertices):
            line = buf.readline()
            v = map(float, line.split())
            mesh.vertices.append( Vertex(v, id) )
        for id in xrange(faces):
            line = buf.readline()
            f = map(int, line.split()[1:])
            vertices = [mesh.vertices[i] for i in f]
            mesh.faces.append( Face(vertices, id) )
        return mesh

scene.register('Mesh', Mesh)
