from defs import *
from numeric import *


class Range(object):
    def __init__(self, a, b):
        self.a, self.b = a, b

    def __repr__(self):
        return '(%g, %g)'%(self.a, self.b)

    def __and__(self, other):
        return Range(max(self.a, other.a), min(self.b, other.b))

    def copy(self):
        return Range(self.a, self.b)

    def get_size(self):
        if self.b < self.a:
            return 0.0
        else:
            return self.b - self.a
    size = property(get_size)


class Box(object):
    def __init__(self, ll, ur):
        self.ll = ll
        self.ur = ur

    @staticmethod
    def compute(mesh):
        M = array(mesh.vectors)
        return Box(M.min(0), M.max(0))


class Ray(object):
    def __init__(self, p, v):
        self.p = p
        self.v = v

    def box_intersection_ranges(self, box):
        (x0, y0, z0), (x1, y1, z1) = box.ll, box.ur
        (px, py, pz), (vx, vy, vz) = self.p, self.v
        if vx >= 0.0:
            Ix = Range((x0-px)/vx, (x1-px)/vx)
        else:
            Ix = Range((x1-px)/vx, (x0-px)/vx)
        if vy >= 0.0:
            Iy = Range((y0-py)/vy, (y1-py)/vy)
        else:
            Iy = Range((y1-py)/vy, (y0-py)/vy)
        if vz >= 0.0:
            Iz = Range((z0-pz)/vz, (z1-pz)/vz)
        else:
            Iz = Range((z1-pz)/vz, (z0-pz)/vz)
        return Ix, Iy, Iz

    def intersect_box(self, box):
        Ix, Iy, Iz = self.box_intersection_ranges(box)
        I = Ix & Iy & Iz
        return I.size > 1e-10

    def intersect_triangle(self, vertices, normal):
        v1, v2, v3 = vertices
        p, v = self.p, self.v
        q, n = vertices[0], normal
        npq = dot(n, q-p)
        nv = dot(n, v)
        EPSILON = 1e-12
        if abs(nv) > EPSILON and abs(npq) > EPSILON:
            t = npq/nv
            if t <= 0:
                return t
            r = p + t*v
            m = array(vertices).T
            if inside(r, vertices):
                return t
            else:
                return None
        else:
            raise 'numeric error'

def area(x, y, z):
    return norm(cross(y-x, z-x))

def inside(p, vertices):
    v1, v2, v3 = vertices
    A = area(v1, v2, v3)
    a = area(p, v2, v3)
    b = area(p, v1, v3)
    c = area(p, v1, v2)
    B = (a+b+c)/A
    if abs(B - 1.0) <= 1e-9:
        return True
    else:
        return False
