from numeric import *
from defs import *
from time import time
from random import random as rand
from objects.mesh import Mesh, Edge
from scene import Scene
from camera import Camera


def scene_from_mesh(filename, size):
    scene = Scene()
    mesh = Mesh.fromBuffer(open(filename))
    scene.objects.append(mesh)
    center, m, M = compute_bounds(mesh)
    d = norm(M-m)
    camera = Camera(center = center-array([0, 0, d]),
                    at = center,
                    up = (0, 1, 0),
                    fov = 90,
                    size = size)
    scene.cameras.append(camera)
    return scene

def compute_bbox(obj):
    vs = array(obj.vectors)
    a, b, c = vs.min(0)
    x, y, z = vs.max(0)
    bbox = Mesh(
        vertices = [
            (a, b, c),
            (a, b, z),
            (a, y, c),
            (a, y, z),
            (x, b, c),
            (x, b, z),
            (x, y, c),
            (x, y, z),
        ],
        faces = [
            (0, 1, 3, 2),
            (4, 5, 7, 6),
            (0, 1, 5, 4),
            (2, 3, 7, 6),
        ]
    )
    return bbox

def compute_mean(mesh):
    vs = array(mesh.vectors)
    return mean(vs, 0)

def compute_bounds(mesh):
    vs = array(mesh.vectors)
    return mean(vs, 0), vs.min(0), vs.max(0)

def compute_face_normals(obj):
    for face in obj.faces:
        v1, v2, v3 = face.vectors
        face.center = array(face.vectors).mean(0)
        n = -cross(v2 - v1, v3 - v1)
        face.normal = n / norm(n)
        if norm(n) < EPSILON: print 'small face normal'

def find_adjacent_faces(obj):
    for face in obj.faces:
        map(lambda v: v.add_face(face), face.vertices)

def compute_vertex_normals(obj):
    for vertex in obj.vertices:
        normals = [face.normal for face in vertex.faces]
        n = mean(array(normals), 0)
        vertex.normal = n / norm(n)

def create_edges(obj):
    Edge.hash_max(len(obj.vertices))

    for triangle in obj.faces:
        vertices = triangle.vertices
        for i in range(3):
            v0, v1 = vertices[i-1], vertices[i]
            id = Edge.hash(v0, v1)
            if id not in obj.edges:
                e = Edge(v0, v1)
                obj.edges[id] = e
            else:
                e = obj.edges[id]
            triangle.add_edge(e)
            e.add_face(triangle)
