import sdl
import defs
from draw import *

NO_EVENT = 0
KEY_DOWN = 2
KEY_UP = 3
MOUSE_MOTION = 4
MOUSE_DOWN = 5
MOUSE_UP = 6
RESIZE = 16

class Viewer(object):
    handler_map = {
        'on_no_event': NO_EVENT,
        'on_key_down': KEY_DOWN,
        'on_key_up': KEY_UP,
        'on_mouse_motion': MOUSE_MOTION,
        'on_mouse_down': MOUSE_DOWN,
        'on_mouse_up': MOUSE_UP,
        'on_resize': RESIZE,
    }

    def __init__(self, size, caption):
        sdl.init()
        sdl.display.set_caption(caption)
        self.flags = sdl.DOUBLEBUF | sdl.RESIZABLE
        self.size = size
        self.surface = sdl.display.set_mode(size, self.flags)
        self.painter = Painter(self.surface.get_size())
        self.handlers = {}
        for attr in dir(self): 
            if self.is_handler(attr):
                self.register_handler(attr)

    def is_handler(self, name):
        return Viewer.handler_map.has_key(name)

    def register_handler(self, name):
        code = Viewer.handler_map[name]
        self.handlers[code] = getattr(self, name)

    def __del__(self):
        sdl.quit()

    def start(self):
        self.loop = True
        while self.loop:
            event = sdl.event.wait()
            if self.handlers.has_key(event.type):
                handler = self.handlers[event.type]
                if event.type == RESIZE:
                    self.resize(event.size)
                handler(event)
            #else:
                #print event

    def resize(self, size):
        self.surface = sdl.display.set_mode(size, self.flags)
        self.painter.resize(size)
        self.size = size

    def stop(self):
        self.loop = False

    def __setattr__(self, name, value):
        if self.is_handler(name):
            import new
            value = new.instancemethod(value, self, self.__class__)
            object.__setattr__(self, name, value)
            self.register_handler(name)
        else:
            object.__setattr__(self, name, value)
